@@ -190,7 +190,7 @@ class ClaudeWrapperV3:
         permission_mode: Optional[str] = None,
         dangerously_skip_permissions: bool = False,
         name: str = "Claude Code",
-        idle_delay: float = 1.0,
+        idle_delay: float = 3.5,
     ):
         # Session management
         self.agent_instance_id = str(uuid.uuid4())
@@ -234,6 +234,9 @@ class ClaudeWrapperV3:
         self.claude_jsonl_path = None
         self.jsonl_monitor_thread = None
         self.running = True
+        # Heartbeat
+        self.heartbeat_thread = None
+        self.heartbeat_interval = 30.0  # seconds
 
         # Claude status monitoring
         self.terminal_buffer = ""
@@ -257,6 +260,25 @@ class ClaudeWrapperV3:
         self.git_tracker: Optional[GitDiffTracker] = None
         self._init_git_tracker()
 
+    def _suspend_for_ctrl_z(self):
+        """Handle Ctrl+Z while in raw mode: restore TTY, stop child and self."""
+        try:
+            self.log("[INFO] Ctrl+Z detected: suspending (raw mode)")
+            if self.original_tty_attrs:
+                termios.tcsetattr(sys.stdin, termios.TCSADRAIN, self.original_tty_attrs)
+            if self.child_pid:
+                try:
+                    os.kill(self.child_pid, signal.SIGTSTP)
+                except Exception as e:
+                    self.log(f"[WARNING] Failed to SIGTSTP child: {e}")
+            # Suspend this wrapper
+            try:
+                os.kill(os.getpid(), signal.SIGTSTP)
+            except Exception as e:
+                self.log(f"[WARNING] Failed to SIGTSTP self: {e}")
+        except Exception as e:
+            self.log(f"[ERROR] Error handling Ctrl+Z: {e}")
+
     def _init_logging(self):
         """Initialize debug logging"""
         try:
@@ -321,6 +343,38 @@ class ClaudeWrapperV3:
             api_key=self.api_key, base_url=self.base_url
         )
 
+    def _heartbeat_loop(self):
+        """Background loop to POST heartbeat while running"""
+        if not self.omnara_client_sync:
+            return
+        session = self.omnara_client_sync.session
+        url = (
+            self.base_url.rstrip("/")
+            + f"/api/v1/agents/instances/{self.agent_instance_id}/heartbeat"
+        )
+        # Small stagger to avoid herd
+        import random
+
+        jitter = random.uniform(0, 2.0)
+        time.sleep(jitter)
+        while self.running:
+            try:
+                resp = session.post(url, timeout=10)
+                if resp.status_code >= 400:
+                    self.log(
+                        f"[WARN] Heartbeat failed {resp.status_code}: {resp.text[:120]}"
+                    )
+            except Exception as e:
+                self.log(f"[WARN] Heartbeat error: {e}")
+            # Sleep interval with small jitter
+            delay = self.heartbeat_interval + random.uniform(-2.0, 2.0)
+            if delay < 5:
+                delay = 5
+            for _ in range(int(delay * 10)):
+                if not self.running:
+                    break
+                time.sleep(0.1)
+
     def get_project_log_dir(self):
         """Get the Claude project log directory for current working directory"""
         cwd = os.getcwd()
@@ -814,7 +868,6 @@ class ClaudeWrapperV3:
 
         # Parent process - handle I/O
         try:
-            # Set stdin to raw mode
             if self.original_tty_attrs:
                 tty.setraw(sys.stdin)
 
@@ -984,6 +1037,15 @@ class ClaudeWrapperV3:
                     try:
                         # Read available data (larger buffer for efficiency)
                         data = os.read(sys.stdin.fileno(), 65536)
+                        if data and b"\x1a" in data:
+                            data = data.replace(b"\x1a", b"")
+                            # Ctrl+Z: suspend child and wrapper
+                            self._suspend_for_ctrl_z()
+                            try:
+                                if self.original_tty_attrs:
+                                    tty.setraw(sys.stdin)
+                            except Exception:
+                                pass
                         if data:
                             # Log user input for debugging
                             try:
@@ -1060,7 +1122,7 @@ class ClaudeWrapperV3:
                             if not hasattr(self, "pending_write_buffer"):
                                 self.pending_write_buffer = b""
 
-                            # Add new data to any pending data
+                            # Add new data to any pending data (post-processed)
                             self.pending_write_buffer += data
 
                             # Try to write as much as possible
@@ -1218,6 +1280,17 @@ class ClaudeWrapperV3:
                 if hasattr(self.message_processor, "last_message_id"):
                     self.message_processor.last_message_id = response.message_id
                     self.message_processor.last_message_time = time.time()
+
+            # Start heartbeat thread
+            try:
+                if not self.heartbeat_thread:
+                    self.heartbeat_thread = threading.Thread(
+                        target=self._heartbeat_loop, daemon=True
+                    )
+                    self.heartbeat_thread.start()
+                    self.log("[INFO] Heartbeat loop started")
+            except Exception as e:
+                self.log(f"[WARN] Failed to start heartbeat loop: {e}")
         except AuthenticationError as e:
             # Log the error
             self.log(f"[ERROR] Authentication failed: {e}")
@@ -1394,8 +1467,8 @@ def main():
     parser.add_argument(
         "--idle-delay",
         type=float,
-        default=1.0,
-        help="Delay in seconds before considering Claude idle (default: 1.0)",
+        default=3.5,
+        help="Delay in seconds before considering Claude idle (default: 3.5)",
     )
 
     # Parse known args and pass the rest to Claude
