@@ -48,9 +48,7 @@ def check_for_updates():
         if latest_version != current_version and current_version != "unknown":
             print(f"\n✨ Update available: {current_version} → {latest_version}")
             print("   Run: pip install --upgrade omnara")
-            print(
-                "   Please keep omnara up-to-date for best compatibility with Claude Code"
-            )
+            print("   Please keep omnara up-to-date for the best experience")
             print("   New versions often include important bug fixes\n")
     except Exception:
         pass
@@ -477,6 +475,12 @@ def run_agent_chat(args, unknown_args):
             "function": "main",
             "argv_name": "amp_wrapper",
         },
+        # 'codex' is implemented as an external binary launcher; handled below
+        "codex": {
+            "module": None,
+            "function": None,
+            "argv_name": "codex",
+        },
     }
 
     # Get agent configuration
@@ -488,8 +492,14 @@ def run_agent_chat(args, unknown_args):
             f"Unknown agent: {agent}. Supported agents: {', '.join(AGENT_CONFIGS.keys())}"
         )
 
-    module = importlib.import_module(config["module"])
-    wrapper_main = getattr(module, config["function"])
+    # Special-case 'codex': spawn the Rust binary with env.
+    if agent == "codex":
+        from omnara.agents.codex import run_codex
+
+        return run_codex(args, unknown_args, api_key)
+
+    module = importlib.import_module(config["module"])  # type: ignore[arg-type]
+    wrapper_main = getattr(module, config["function"])  # type: ignore[index]
 
     # Prepare sys.argv for the wrapper
     original_argv = sys.argv
@@ -619,7 +629,7 @@ def add_global_arguments(parser):
     )
     parser.add_argument(
         "--agent",
-        choices=["claude", "amp"],
+        choices=["claude", "amp", "codex"],
         default="claude",
         help="Which AI agent to use (default: claude)",
     )
@@ -641,8 +651,8 @@ def add_global_arguments(parser):
     parser.add_argument(
         "--idle-delay",
         type=float,
-        default=1.0,
-        help="Delay in seconds before considering Claude idle (default: 1.0)",
+        default=3.5,
+        help="Delay in seconds before considering Claude idle (default: 3.5)",
     )
 
 
